@@ -1,19 +1,25 @@
-//Copyright (C) 1997-2004 ZSNES Team ( zsknight@zsnes.com / _demo_@zsnes.com )
-//
-//This program is free software; you can redistribute it and/or
-//modify it under the terms of the GNU General Public License
-//as published by the Free Software Foundation; either
-//version 2 of the License, or (at your option) any later
-//version.
-//
-//This program is distributed in the hope that it will be useful,
-//but WITHOUT ANY WARRANTY; without even the implied warranty of
-//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//GNU General Public License for more details.
-//
-//You should have received a copy of the GNU General Public License
-//along with this program; if not, write to the Free Software
-//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+/*
+Copyright (C) 1997-2005 ZSNES Team ( zsKnight, _Demo_, pagefault, Nach )
+
+http://www.zsnes.com
+http://sourceforge.net/projects/zsnes
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either
+version 2 of the License, or (at your option) any later
+version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 
 
 #include <stdio.h>
@@ -261,10 +267,6 @@ DWORD ZFileMKDir()
 #endif
 }
 
-extern char SRAMDir;
-extern char InitDir;
-extern char LoadDir;
-
 
 DWORD ZFileCHDir()
 {
@@ -456,38 +458,46 @@ DWORD GetDate()
    return(value);
 }
 
+extern char SRAMDir[1024];
+extern char LoadDir[512];
+
 #ifdef __LINUX__
 
+char zcfgdir[1024];
+#define ZCFG_DIR "/.zsnes"
+#define ZCFG_DIR_LEN (1023-strlen(ZCFG_DIR))
 
 void obtaindir()
 {
-	char *cfgdir = NULL;
-	char *homedir = NULL;
-	DIR *tmp;
+  char *homedir = 0;
+  DIR *tmp;
 
-	if ((homedir = (char *)getenv("HOME"))==NULL) {
-		homedir = (char *)malloc(128);
-		getcwd(homedir, 128);
-	}
-	cfgdir = (char *)malloc(strlen(homedir)+strlen("/.zsnes")+1);
-	strcpy(cfgdir, homedir);
-	strcat(cfgdir, "/.zsnes");
-	tmp = opendir(cfgdir);
-	if (tmp == NULL) {
-		MKPath = cfgdir;
-		ZFileMKDir();
-	} else {
-		closedir(tmp);
-	}
-//	strcpy(&InitDir, cfgdir);
-//	getcwd(&InitDir, 128);
-	if (SRAMDir == 0){
-		strcpy(&SRAMDir, cfgdir);
-	}
-	free(cfgdir);
-	if (LoadDir == 0) {
-		getcwd(&LoadDir, 128);
-	}
+  if ((homedir = (char *)getenv("HOME")) == 0)
+  {
+    homedir = (char *)malloc(ZCFG_DIR_LEN);
+    getcwd(homedir, ZCFG_DIR_LEN);
+  }
+  strcpy(zcfgdir, homedir);
+  free(homedir);
+  strcat(zcfgdir, ZCFG_DIR);
+  tmp = opendir(zcfgdir);
+  if (tmp == NULL) 
+  {
+    MKPath = zcfgdir;
+    ZFileMKDir();
+  } 
+  else 
+  {
+    closedir(tmp);
+  }
+  if (*SRAMDir == 0)
+  {
+    strcpy(SRAMDir, zcfgdir);
+  }
+  if (*LoadDir == 0) 
+  {
+    getcwd(LoadDir, 512);
+  }
 }
 
 extern char fnamest, fnames;
@@ -531,3 +541,5 @@ void popdir()
 }
 #endif
 
+
+
