@@ -1,5 +1,9 @@
 /*
-Copyright (C) 1997-2004 ZSNES Team ( zsknight@zsnes.com / _demo_@zsnes.com )
+Copyright (C) 1997-2005 ZSNES Team ( zsKnight, _Demo_, pagefault, Nach )
+
+http://www.zsnes.com
+http://sourceforge.net/projects/zsnes
+
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
@@ -90,14 +94,11 @@ int my_atoi(char *nptr) {
 	return atoi(nptr);
 }
 
-extern void UnProtectMemory (void);
-
 int main (int argc, char *argv[]) {
 	int opt,p,pp;
 
 	char *fvar;
 
-	UnProtectMemory();
 	while((opt = getopt(argc, argv, "01:2:789a:c:d:ef:g:hijk:lmno:p:r:s:tuv:wyz?")) != -1) {
 		switch(opt) {
 			/* Palette 0 disable */
